/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.awt.Color;
import java.awt.Font;
import java.awt.Toolkit;
import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.JTextField;
import javax.swing.JScrollPane;
import javax.swing.JScrollBar;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.BadLocationException;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.util.Vector;

public class ChatPanel extends JPanel implements Runnable {

    private JTextPane chatBox;
    private JTextField chatField;
    private JScrollPane scrollPane;
    private boolean autoScroll = true;

    String text = null;

    Font smallFont;
    
    private Toolkit toolkit = Toolkit.getDefaultToolkit();

    //  keep a history of all the chat messages sent
    Vector chats = new Vector();

    int currentChat = 0;

    //  parent
    Q3ServerKit q3Gui;

    //  formatted text of the chat area
    StyledDocument doc;
    
    //  the format of the text in the chat area
    SimpleAttributeSet coloredFont;

    public ChatPanel(Q3ServerKit q3Gui) {
	this.q3Gui = q3Gui;
	setLayout(new PercentLayout());
	setBackground(Color.black);
	setBorder(BorderFactory.createLineBorder(Color.white));
	smallFont = new Font("plain", Font.PLAIN, 10);
	chatBox = new JTextPane(new DefaultStyledDocument());
	chatBox.setForeground(Color.white);
	chatBox.setBackground(Color.black);
	chatBox.setEditable(false);
	chatBox.setFont(smallFont);
	scrollPane = new JScrollPane(chatBox);

	chatField = new JTextField();
	chatField.addActionListener(new ChatListener());
	chatField.addKeyListener(new ChatKeyListener());
	add("0 0 100 80", scrollPane);
	add("2 82 98 98", chatField);
    }
	
    //  called external to this class to set new text 
    //  to be displayed from players on the server...
    public void addText(String text) {
	this.text = text;
	try {
	    SwingUtilities.invokeLater(this);
	} catch (Exception e) {
	    db("Problem updating chat panel...");
	}
    }
    
    //  run in the event dispatch thread to display new text
    public void run() {
	doc = chatBox.getStyledDocument();

	//  make sure it isn't too big...
	//  if so, clear off the first few lines
	//  (causes out of memory errors otherwise)
	int length = doc.getLength();
	if(length > 5000) {
	    try {
		doc.remove(0, 1000);
	    } catch (BadLocationException ble) {
		db("problem erasing chat screen");
	    }
	}
	
	coloredFont = new SimpleAttributeSet();
	String name = text.substring(0, text.indexOf(':'));
	name = name.trim();
	//  display one color for the bots chat
	//  and another for the real people chat :)
	if(q3Gui.botsPanel.botModel.contains(name))
	    StyleConstants.setForeground(coloredFont, Color.white);
	else 
	    StyleConstants.setForeground(coloredFont, Color.green);
	StyleConstants.setFontFamily(coloredFont, "SansSerif");
	try {
	    doc.insertString(doc.getLength(), text + "\n", coloredFont);
	    if(autoScroll) {
		JScrollBar bar = scrollPane.getVerticalScrollBar();
		if(bar != null)
		    bar.setValue(bar.getMaximum());
	    }
	} catch(BadLocationException ble) {
	    db("problem writing to chat screen");
	}
    }
    
    //  debug printing util
    private void db(String s) {
	System.err.println(s);
    }
    
    //  sleep util
    private void sleep(int i) {
	try {
	    Thread.sleep(i);
	} catch (InterruptedException ie) {
	    db("Error sleeping");
	}
    }

    public void setAutoScroll(boolean scrollIt) {
	autoScroll = scrollIt;
    }
    
    
    //  scrolls the chat field like a UNIX command prompt
    //  the argument is either 0 for back or 1 for forward
    protected void scrollChatEntry(int direction) {
	if(direction < 1) {
	    if(currentChat == 1) {
		chatField.setText((String) chats.elementAt(0));
		chats.removeElementAt(--currentChat);
		return;
	    }
	    if((currentChat - 1) < 0) {
		toolkit.beep();
		return;
	    }
	    chatField.setText((String) chats.elementAt(--currentChat));
	} else {
	    if(currentChat == 0)
		chats.insertElementAt(chatField.getText(), 0);
	    if((currentChat + 1) >= chats.size()) {
		toolkit.beep();
		return;
	    }
	    chatField.setText((String) chats.elementAt(++currentChat));
	}
    }

    //  listens for a chat post
    class ChatListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    String message = chatField.getText();
	    doc = chatBox.getStyledDocument();
	    //  check for a clear request
	    if(message.equals("/clear")) {
		try {
		    doc.remove(0, doc.getLength());
		} catch(BadLocationException ble) {
		    db("problem clearing chat screen");
		} 
		chats.insertElementAt(message, 0);
		chatField.setText("");
		return;
	    }
	    synchronized (doc) {
		coloredFont = new SimpleAttributeSet();
		StyleConstants.setForeground(coloredFont, Color.yellow);
		StyleConstants.setFontFamily(coloredFont, "SansSerif");
		try {
		    doc.insertString(doc.getLength(), "console: " + message 
				     + "\n", coloredFont);
		    if(autoScroll) {
			JScrollBar bar = scrollPane.getVerticalScrollBar();
			if(bar != null)
			    bar.setValue(bar.getMaximum());
		    }
		    
		} catch(BadLocationException ble) {
		    db("problem writing to chat screen");
		} 
	    } 
	    q3Gui.q3Client.send("say " + message);
	    chats.insertElementAt(message, 0);
	    chatField.setText("");
	}
    }

    //  handles scrolling to previous commands
    class ChatKeyListener implements KeyListener {
	/** Handle the key typed event from the text field. */
	public void keyTyped(KeyEvent e) {
	}
	
	/** Handle the key pressed event from the text field. */
	public void keyPressed(KeyEvent e) {
	    int keyCode = e.getKeyCode();
	    if(keyCode == 40)
		scrollChatEntry(0);
	    if(keyCode == 38)
		scrollChatEntry(1);
	}
	
	/** Handle the key released event from the text field. */
	public void keyReleased(KeyEvent e) {
	}

    }

}

