/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;


import java.awt.Color;
import java.awt.Font;
import java.awt.Toolkit;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JTextPane;
import javax.swing.JTextField;
import javax.swing.JScrollPane;
import javax.swing.JScrollBar;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.BadLocationException;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.util.Vector;
import java.awt.Dimension;


public class Console extends JFrame implements Runnable {

    private JPanel consolePanel;
    private JTextPane console;
    private JTextField chatField;
    private JScrollPane scrollPane;
    private boolean autoScroll = true;

    String text = null;
    Vector messages = null;

    Font smallFont;
    
    private Toolkit toolkit = Toolkit.getDefaultToolkit();

    //  keep a history of all the chat messages sent
    Vector chats = new Vector();

    int currentChat = 0;

    //  parent
    Q3ServerKit q3Gui;

    //  formatted text of the chat area
    StyledDocument doc;
    
    //  the format of the text in the chat area
    SimpleAttributeSet coloredFont;

    public Console(Q3ServerKit q3Gui) {
	this.q3Gui = q3Gui;

	consolePanel = new JPanel();
	consolePanel.setLayout(new PercentLayout());
	consolePanel.setBackground(Color.black);
	consolePanel.setBorder(BorderFactory.createLineBorder(Color.white));
	smallFont = new Font("plain", Font.PLAIN, 10);
	console = new JTextPane(new DefaultStyledDocument());
	console.setForeground(Color.white);
	console.setBackground(Color.black);
	console.setEditable(false);
	console.setFont(smallFont);
	scrollPane = new JScrollPane(console);

	chatField = new JTextField();
	chatField.addActionListener(new ChatListener());
	chatField.addKeyListener(new ChatKeyListener());
	consolePanel.add("0 0 100 80", scrollPane);
	consolePanel.add("2 82 98 98", chatField);
	this.getContentPane().add(consolePanel);
	Dimension dimension = new Dimension(400, 250);
	this.setSize(dimension);
	this.setBackground(Color.black);

	//  initialize the messsage buffer
	messages = new Vector();
    }
	
    //  called external to this class to set new text 
    //  to be displayed from players on the server...
    public void addText(String text) {
	synchronized(messages) {
	    messages.addElement(text);
	}
	try {
	    SwingUtilities.invokeLater(this);
	} catch (Exception e) {
	    db("Problem updating chat panel...");
	}
    }
    
    //  run in the event dispatch thread to display new text
    public void run() {
	while(!messages.isEmpty()) {
	    //  get the current console buffer
	    doc = console.getStyledDocument();
	    coloredFont = new SimpleAttributeSet();

	    //  make sure it isn't too big...
	    //  if so, clear off the first few lines
	    //  (causes out of memory errors otherwise)
	    int length = doc.getLength();
	    if(length > 5000) {
		try {
		    doc.remove(0, 2000);
		} catch (BadLocationException ble) {
		    db("problem erasing chat screen");
		}
	    }

	    //  grab the first line
	    String line;
	    synchronized (messages) {
		line = (String) messages.elementAt(0);
		messages.removeElementAt(0);
	    }
	    line = line.trim();

	    //  change the text color for some of the different messages
	    StyleConstants.setForeground(coloredFont, Color.white);
	    if(line.startsWith("Kill"))
		StyleConstants.setForeground(coloredFont, Color.red);

	    //  change the color for chats
	    if(line.startsWith("say")) {
		String temp = line.substring(line.indexOf(':') + 1);
		String name = temp.substring(0, temp.indexOf(':'));
		name = name.trim();
		//  set to green for player chats
		//  white for bot chats
		if(q3Gui.botsPanel.botModel.contains(name))
		    StyleConstants.setForeground(coloredFont, Color.white);
		else 
		    StyleConstants.setForeground(coloredFont, Color.green);
	    }
		
	    //  change the color of the admin messages
	    if(line.startsWith("console"))
		StyleConstants.setForeground(coloredFont, Color.yellow);

	    //  change the color of items
	    if(line.startsWith("Item"))
		StyleConstants.setForeground(coloredFont, Color.blue);

	    //  change the color of the end of game scores
	    if(line.startsWith("score"))
		StyleConstants.setForeground(coloredFont, Color.orange);

	    //StyleConstants.setForeground(coloredFont, Color.green);
	    StyleConstants.setFontFamily(coloredFont, "SansSerif");
	    try {
		doc.insertString(doc.getLength(), line + "\n", coloredFont);
		if(autoScroll) {
		    JScrollBar bar = scrollPane.getVerticalScrollBar();
		    if(bar != null)
			bar.setValue(bar.getMaximum());
		}
	    } catch(BadLocationException ble) {
		db("problem writing to chat screen");
	    }
	}
    }
    
    //  debug printing util
    private void db(String s) {
	System.err.println(s);
    }
    
    //  sleep util
    private void sleep(int i) {
	try {
	    Thread.sleep(i);
	} catch (InterruptedException ie) {
	    db("Error sleeping");
	}
    }

    public void setAutoScroll(boolean scrollIt) {
	autoScroll = scrollIt;
    }
    
    
    //  scrolls the chat field like a UNIX command prompt
    //  the argument is either 0 for back or 1 for forward
    protected void scrollChatEntry(int direction) {
	if(direction < 1) {
	    if(currentChat == 1) {
		chatField.setText((String) chats.elementAt(0));
		chats.removeElementAt(--currentChat);
		return;
	    }
	    if((currentChat - 1) < 0) {
		toolkit.beep();
		return;
	    }
	    chatField.setText((String) chats.elementAt(--currentChat));
	} else {
	    if(currentChat == 0)
		chats.insertElementAt(chatField.getText(), 0);
	    if((currentChat + 1) >= chats.size()) {
		toolkit.beep();
		return;
	    }
	    chatField.setText((String) chats.elementAt(++currentChat));
	}
    }

    //  listens for a chat post
    class ChatListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    String message = chatField.getText();
	    doc = console.getStyledDocument();
	    //  check for a clear request
	    if(message.equals("/clear")) {
		try {
		    doc.remove(0, doc.getLength());
		} catch(BadLocationException ble) {
		    db("problem clearing chat screen");
		} 
		chats.insertElementAt(message, 0);
		chatField.setText("");
		return;
	    }
	    if(message.startsWith("say")) 
		addText("console: " + message.substring(3));
	    q3Gui.q3Client.send(message);
	    chats.insertElementAt(message, 0);
	    chatField.setText("");
	}
    }

    //  handles scrolling to previous commands
    class ChatKeyListener implements KeyListener {
	/** Handle the key typed event from the text field. */
	public void keyTyped(KeyEvent e) {
	}
	
	/** Handle the key pressed event from the text field. */
	public void keyPressed(KeyEvent e) {
	    int keyCode = e.getKeyCode();
	    if(keyCode == 40)
		scrollChatEntry(0);
	    if(keyCode == 38)
		scrollChatEntry(1);
	}
	
	/** Handle the key released event from the text field. */
	public void keyReleased(KeyEvent e) {
	}

    }

}

