/*
 * Copyright (C) years Free Software Foundation, Inc.
 * Brian Satterfield <neuralnets@home.net>, 2000.
 * Chris Garret
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.awt.*;

import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.JScrollPane;
import javax.swing.JScrollBar;
import javax.swing.text.*;
import javax.swing.BorderFactory;

public class InfoPanel extends JPanel {

  protected JTextPane infoBox;
  protected JScrollPane scrollPane;
  protected boolean autoScroll = true;
  
  
  public InfoPanel() {
      setLayout(new PercentLayout());
      setBorder(BorderFactory.createCompoundBorder(
        BorderFactory.createEtchedBorder(),
        BorderFactory.createEmptyBorder(2, 2, 2, 2)));

      infoBox = new JTextPane(new DefaultStyledDocument());
      infoBox.setForeground(Color.white);
      infoBox.setBackground(Color.black);
      infoBox.setEditable(false);
      
      scrollPane = new JScrollPane(infoBox);
      
      add("0 0 100 100", scrollPane);

  }
  
  public void setAutoScroll(boolean scrollIt) {
    autoScroll = scrollIt;
  }
  
  public void clear() {
      StyledDocument doc = infoBox.getStyledDocument();
      try {
	  doc.remove(0, doc.getLength());
      } catch (BadLocationException ble) {
	  System.err.println("Caught Bad Location Exception (clear)\n");
      }
  }

  public void replace(String s, Color color) {
      StyledDocument doc = infoBox.getStyledDocument();
      try {
	  infoBox.invalidate();
	  doc.remove(0, doc.getLength());
	  displayInfo(s, color);
	  infoBox.repaint();
	  infoBox.validate();
      } catch (BadLocationException ble) {
	  System.err.println("Caught Bad Location Exception (clear)\n");
      }
  }

  public void displayInfo(String info, Color color) {

    if(infoBox != null) {

      SimpleAttributeSet coloredFont = new SimpleAttributeSet();
      StyleConstants.setForeground(coloredFont, color);
      StyleConstants.setFontFamily(coloredFont, "SansSerif");
      StyledDocument doc = infoBox.getStyledDocument();
      
      try {
        
        doc.insertString(doc.getLength(), info + "\n", coloredFont);
        if(autoScroll) {
          JScrollBar bar = scrollPane.getVerticalScrollBar();
          if(bar != null)
            bar.setValue(bar.getMaximum());
        }
        
      } catch(BadLocationException ble) {
        
        System.err.println("Caught Bad Location Exception (displayInfo)\n");
        
      }

      if(!infoBox.isVisible())
        System.out.println(info);

    } else {

      System.out.println(info);

    }

  }

  public void appendInfo(String info, Color color) {

    if(infoBox != null) {

      SimpleAttributeSet coloredFont = new SimpleAttributeSet();
      StyleConstants.setForeground(coloredFont, color);
      StyleConstants.setFontFamily(coloredFont, "SansSerif");
      StyledDocument doc = infoBox.getStyledDocument();
      
      try {
        
        doc.insertString(doc.getLength(), info, coloredFont);
        if(autoScroll) {
          JScrollBar bar = scrollPane.getVerticalScrollBar();
          if(bar != null)
            bar.setValue(bar.getMaximum());
        }
        
      } catch(BadLocationException ble) {
        
        System.err.println("Caught Bad Location Exception (appendInfo)\n");
        
      }

      if(!infoBox.isVisible())
        System.out.println(info);

    } else {

      System.out.println(info);

    }

  }

  public void replaceInfo(String info, Color color) {

    if(infoBox != null) {

      SimpleAttributeSet coloredFont = new SimpleAttributeSet();
      StyleConstants.setForeground(coloredFont, color);
      StyleConstants.setFontFamily(coloredFont, "SansSerif");
      StyledDocument doc = infoBox.getStyledDocument();
      
      try {
	doc.remove(0, doc.getLength());
	doc.insertString(doc.getLength(), info + "\n", coloredFont);
        if(autoScroll) {
          JScrollBar bar = scrollPane.getVerticalScrollBar();
          if(bar != null)
            bar.setValue(bar.getMaximum());
        }
        
      } catch(BadLocationException ble) {
        
        System.err.println("Caught Bad Location Exception (replaceInfo)\n");
        
      }

      if(!infoBox.isVisible())
        System.out.println(info);

    } else {

      System.out.println(info);

    }

  }
}

