/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import javax.swing.JPanel;
import javax.swing.JList;
import javax.swing.JButton;
import javax.swing.JTextField;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.SwingUtilities;
import javax.swing.DefaultListModel;
import javax.swing.ListSelectionModel;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Color;
import java.util.Vector;

public class MapsPanel extends JPanel implements Runnable {

    JTextField directory;
    JLabel dirLabel;

    JList mapList;
    DefaultListModel mapModel;

    JButton scanButton;
    JButton changeButton;

    JScrollPane scrollPane;

    Vector maps;

    //  parent
    Q3ServerKit q3Gui;

    public MapsPanel(Q3ServerKit q3Gui) {
	this.q3Gui = q3Gui;
	
	setLayout(new PercentLayout());
	dirLabel = new JLabel("Map Directory:");
	dirLabel.setForeground(Color.white);
	directory = new JTextField("maps/*");
	directory.addActionListener(new DirectoryListener());
	mapModel = new DefaultListModel();
	mapList = new JList(mapModel);
	mapList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	scrollPane = new JScrollPane(mapList);
	
	scanButton = new JButton("scan");
	scanButton.setBackground(Color.black);
	scanButton.setForeground(Color.white);
	scanButton.addActionListener(new ScanButtonListener());
	changeButton = new JButton("change");
	changeButton.setBackground(Color.black);
	changeButton.setForeground(Color.white);
	changeButton.addActionListener(new ChangeButtonListener());

	//  will hold a list of the maps
	maps = new Vector();
	
	setBackground(Color.black);

	add(dirLabel, "0 0 100 10");
	add(directory, "0 12 100 25");
	add(scrollPane, "0 30 100 85");
	add(scanButton, "0 88 43 100");
	add(changeButton, "45 88 100 100");
    }

    //  this code will be run in the event dispatch thread
    //  used to rescan the maps dir and re-populate the list
    public void run() {
	int size = mapModel.getSize();
	for(int i = 0; i < size; i++) {
	    mapModel.remove(0);
	}
	size = maps.size();
	for(int i = 0; i < size; i++) {
	    mapModel.addElement((String) maps.elementAt(i));
	}
    }

    //  parse out any maps in the server response
    private void parseMaps(String serverResponse) {
	//  clear out the old maps
	maps.clear();

	//  make easier to parse
	serverResponse = serverResponse + "\n";
	String line;
	int newLine = serverResponse.indexOf("\n");
	while(newLine >= 0) {
	    line = serverResponse.substring(0, newLine);
	    serverResponse = serverResponse.substring(newLine + 1);
	    newLine = serverResponse.indexOf("\n");
	    if(line.endsWith("bsp")) {
		line = line.substring(0, line.indexOf('.'));
		maps.addElement(line);
	    }
	}
    }

    //  listens for a scan request
    class ScanButtonListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    String dir = directory.getText();
	    String response = q3Gui.q3Client.send("dir " + dir);
	    if(response.startsWith("Bad")) {
		q3Gui.effectsPanel.setText("Incorrect password...");
		//  switch to the password command...
		q3Gui.newAction("pass");
		return;
	    }
	    MapsPanel thisPanel 
		= (MapsPanel) ((JButton) e.getSource()).getParent();
	    parseMaps(response);
	    try {
		SwingUtilities.invokeLater(thisPanel);
	    } catch (Exception ex) {
		db("Problem changing panels...");
	    }
	}
    }
    
    //  listens for a request to change the map
    class ChangeButtonListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    if(mapList.getSelectedIndex() < 0) {
		q3Gui.effectsPanel.setText("Please select a map...");
		return;
	    }
	    String mapName = (String) mapList.getSelectedValue();
	    String response = q3Gui.q3Client.send("map " + mapName);
	    q3Gui.effectsPanel.setText(response);
	}
    }

    //  listens for a new directory/scan request
    class DirectoryListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    String dir = ((JTextField) e.getSource()).getText();
	    String response = q3Gui.q3Client.send("dir " + dir);
	    if(response.startsWith("Bad")) {
		q3Gui.effectsPanel.setText("Incorrect password...");
		//  switch to the password command...
		q3Gui.newAction("pass");
		return;
	    }
	    MapsPanel thisPanel 
		= (MapsPanel) ((JTextField) e.getSource()).getParent();
	    parseMaps(response);
	    try {
		SwingUtilities.invokeLater(thisPanel);
	    } catch (Exception ex) {
		db("Problem changing panels...");
	    }
	}
    }

    private void db(String s) {
	System.err.println(s);
    }
    
}
