/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.awt.Dimension;
import java.awt.Color;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JTable;
import javax.swing.JScrollPane;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JPanel;
import javax.swing.table.DefaultTableModel;
import java.util.Vector;
import javax.swing.SwingUtilities;

public class PlayerDisplay extends JPanel implements Runnable {

    private JTable table;
    private DefaultTableModel tableModel;
    private Vector columns;
    private Vector playerInfo;
    private JScrollPane scrollPane;
    private static final String botString = "bots";
    private static final String kickString = "kick";
    private JButton kickButton;

    protected Q3ServerKit q3Gui;
    protected Q3InfoClient q3Client;

    //  a count of the rows in the table 
    private int rows = 0;

    //  a count of the new rows
    private int newRows = 0;

    //  counter
    private int i = 0;

    //  generic player to add to the table
    Player player = null;


    public PlayerDisplay(Q3ServerKit q3Gui) {
	this.q3Gui = q3Gui;
	this.q3Client = q3Gui.q3Client;
	this.setLayout(new PercentLayout());
	this.setBackground(Color.black);

        //Create the table
	columns = new Vector();
	columns.addElement("Player");
	columns.addElement("Ping");
	columns.addElement("Score");
	
	tableModel = new DefaultTableModel(columns, 0);
	table = new JTable(tableModel);
	table.setPreferredScrollableViewportSize(new Dimension(20, 20));
	scrollPane = new JScrollPane(table);

        JButton botButton = new JButton(botString);
        botButton.setActionCommand(botString);
        botButton.addActionListener(new BotListener());
	botButton.setBackground(Color.black);
	botButton.setForeground(Color.white);

        kickButton = new JButton(kickString);
        kickButton.setActionCommand(kickString);
        kickButton.addActionListener(new KickListener());
	kickButton.setEnabled(false);
	kickButton.setBackground(Color.black);
	kickButton.setForeground(Color.white);

        this.add(scrollPane, "0 0 100 83");
        this.add(botButton, "0 85 49 100");
        this.add(kickButton, "51 85 100 100");
	
    }

    public void setPlayers(Vector players) {
	this.playerInfo = players;
	try {
	    SwingUtilities.invokeLater(this);
	} catch (Exception e) {
	    db("Problem updating gui...");
	}
    }

    public void run() {
	//  clear the table
	rows = tableModel.getRowCount();
	for(i = 0; i < rows; i++) {
	    tableModel.removeRow(0);
	}

	//  add the new rows
	newRows = playerInfo.size();
	for(i = 0; i < newRows; i++) {
	    player = (Player) playerInfo.elementAt(i);
	    tableModel.addRow(player.getPlayer());
	}

	this.repaint();

	//  get the new count
	rows = tableModel.getRowCount();
	if(rows > 0) {
	    //  detected players...enable the kick button
	    kickButton.setEnabled(true);
	} else {
	    //  Nobody's left, disable kicking.
	    kickButton.setEnabled(false);
	}
    }

    class KickListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
	    int selectedRow = table.getSelectedRow();
	    if(selectedRow < 0) {
		q3Gui.effectsPanel.setText("Please select a player first...");
		return;
	    }
	    String playerName = table.getValueAt(selectedRow, 0).toString();
	    String response = q3Client.send("kick " + playerName);
	    if(response.indexOf("password") > 0) {
		q3Gui.effectsPanel.setText("Invalid password");
		//  switch to the password command...
		q3Gui.newAction("pass");
		return;
	    } 
	    if(response.indexOf("kicked") > 0) 
		q3Gui.effectsPanel.setText(playerName + " was kicked.");
        }
    }

    //  just changes the panel to the bots...
    class BotListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
	    q3Gui.newAction("bots");
        }
    }

    private void db(String s) {
	System.err.println(s);
    }
}
