/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.DatagramPacket;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.io.IOException;
import java.util.Vector;

public class Q3InfoClient {

    DatagramSocket ds;
    DatagramPacket dp;
    InetAddress ia;
    String info;
    String host = null;
    int port = 27960;
    String password = "";

    //  possible commands
    //
    //    - getstatus
    //    - getinfo
    //    - getchallenge
    //    - connect
    //    - rcon
    public Q3InfoClient() {
	
    }

    public boolean serverDetected() {
	String response = getInfo("xxxxgetinfo");
	if(response == null) 
	    return false;
	else 
	    return true;
    }

    public void setHost(String host) {
	this.host = host;
    }
    
    public void setPort(String port) {
	try {
	    this.port = Integer.parseInt(port.trim());
	}   catch (NumberFormatException nfe) {
	    //  will use default port;
	}
    }
	
    //  use this to create the datagram socket ONCE
    //  then in getInfo just connect/disconnect
    public boolean connect() {
	//  if the host is set, use that 
	//  otherwise use the localhost
	ia = null;
	try {
	    if(host == null) {
		ia = InetAddress.getLocalHost();
		host = ia.getHostAddress();
	    } else { 
		ia = InetAddress.getByName(host);
	    }
	} catch (UnknownHostException uhe) {
	    db("Unknown host.");
	    return false;
	}
	//  to connect and send a command
	try {
	    ds = new DatagramSocket();
	    //  anything over a 3 second ping is really insane...
	    ds.setSoTimeout(3000);
	} catch (SocketException se) {
	    db("Cannot create socket connection");
	    se.printStackTrace();
	    return false;
	}
	return true;
    }

    public void setPassword(String password) {
	this.password = password;
    }
    
    synchronized public String send(String command) {
	String playername = "";
	String mapName = "";
	if(command.startsWith("addbot")) {
	    int spaces = 0;
	    String commandCopy = command;
	    while(commandCopy.indexOf(' ') > 0) {
		commandCopy = commandCopy.substring(commandCopy.indexOf(' ') 
						    + 1);
		spaces++;
	    }
	    if(spaces < 2) {
		String response = "Missing bot name or skill level. \n"
		                   + "Usage: addbot bot-name skill[1-4]";
		return response;
	    }
	    playername = command.substring(command.indexOf(" ") + 1);
	    playername = playername.substring(0, playername.indexOf(" "));
	    playername = playername.trim();
	}
	if(command.startsWith("kick")) {
	    playername = command.substring(command.indexOf(" ") + 1);
	    playername = playername.trim();
	}
	if(command.startsWith("map") || command.startsWith("devmap")) {
	    mapName = command.substring(command.indexOf(" ") + 1);
	    mapName = mapName.trim();
	}
	if(password.length() > 0) 
	    command = password + " " + command;
	String response = getInfo("xxxxrcon" + " " + command);
	if(response == null && mapName.length() > 0) 
	    response = mapName + " was started";
	if(response == null) 
	    response = "Could not contact server.";
	response = response.substring(response.indexOf("\n") + 1);
	response = response.trim();
	if(response.indexOf("entered") > 0 && playername.length() > 0)
	    response = playername + " entered the game.";
	if(response.indexOf("kicked") > 0)
	    response = playername + " was kicked.";
	if(response.indexOf("Game Initialization") > 0) 
	    response = mapName + " was started";
	return response;
    }

    synchronized public String getInfo(String command) {
	//  create the socket
	try {
	    ds = new DatagramSocket();
	    ds.setSoTimeout(3000);
	} catch (SocketException se) {
	    db("Cannot create socket connection");
	    se.printStackTrace();
	    return null;
	}
	//  set up the message
	byte [] buff = command.getBytes();
	buff[0] = (byte)0xff; // oob
	buff[1] = (byte)0xff;
	buff[2] = (byte)0xff;
	buff[3] = (byte)0xff;
	dp = new DatagramPacket(buff, buff.length, ia, port);
	try {
	    ds.send(dp);
	} catch (IOException ioe) {
	    db("Could not send packets.");
	    return null;
	}

	// to receive
	buff = new byte[65507];
	dp = new DatagramPacket(buff, buff.length);
	try {
	    ds.receive(dp);
	}catch (IOException ioe) {
	    db("Could not receive packets.");
	    return null;
	}
	ds.close();
	info = null;
	info = new String(buff);
	return info;
    }
	
    private void db(String s) {
	System.err.println(s);
    }
	


}

