/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.util.Vector;
import javax.swing.SwingUtilities;

public class Q3Server implements Runnable {

    int delay = 5000;
    Q3ServerKit q3Gui;
    Q3InfoClient q3Client;
    int bots = 0;

    //  server info stuff
    String map =  "";
    String maxClients = "";
    String serverName = "";
    String timeLimit = "";
    String fragLimit = "";
    Vector serverInfo = new Vector();

    //  collection of all players, 
    //  from highest score to lowest
    Vector players;

    //  history of the players
    Vector playerHistory = new Vector();
    
    //  indicates whether the game is in progress
    boolean active = true;

    //  indicate whether the server is empty or not
    boolean empty = false;
    
    //  used to indicate whether we are waiting for a server
    boolean waiting = false;

    //  used to determine whether the serverinfo has changed
    boolean newInfo = false;

    //  used to determine whether there are any 'real' players
    boolean foundRealPlayer = false;

    //  debug
    int count = 0;

    //  holds the response of the server
    String response = null;

    //  one line of the response 
    String line = null;

    //  the player part of response
    String playerLine = null;
    
    //  the score part of the response
    String score = null;

    //  the ping part of the response
    String ping = null;

    //  the name of the player in the response
    String name = null;

    //  the player created with the information above
    Player player = null;
    
    //  the next player in the list when we sort them
    Player nextPlayer = null;

    //  the winner of the game
    Player winner = null;

    //  the runtime of the system
    Runtime runtime = Runtime.getRuntime();

    //  stats thread
    public Statistics statistics;

    public Q3Server(Q3ServerKit q3Gui) {
	this.q3Gui = q3Gui;
	q3Client = q3Gui.q3Client;
	startStats();
    }

    //  starts the stats thread
    private void startStats() {
	if(statistics == null) {
	    statistics = new Statistics(this);
	    new Thread(statistics).start();
	}
    }

    synchronized public String getMap() {
	return map;
    }
    
    synchronized public String getMaxClients() {
	return maxClients;
    }

    synchronized public String getServerName() {
	return serverName;
    }

    synchronized public String getTimeLimit() {
	return timeLimit;
    }

    synchronized public String getFragLimit() {
	return fragLimit;
    }

    synchronized public Vector getPlayers() {
	return playerHistory;
    }

    synchronized public void setDelay(int delay) {
	this.delay = delay;
    }

    synchronized public boolean isActive() {
	return active;
    }

    public void run() {
	while(true) {
	    synchronized (q3Client) {
		//  get the server info
		response = q3Client.getInfo("xxxxgetstatus");
	    }
	    //  take care of a null response
	    if(response == null && waiting == false)
		q3Gui.effectsPanel.setFadeText("Lost contact with server.");

	    //  take care of a shutdown broadcast
	    if(response != null && response.indexOf("shutdown") > 0) {
		q3Gui.effectsPanel.setFadeText("Server restarting.");
		response = null;
	    }
		
	    //  make sure this is a status response
	    //  this is causing a problem!!!!
	    if(response != null && response.indexOf("statusResponse") < 0)
		response = null;

	    //  if no server found...ping and sleep
	    while(response == null) {
		waiting = true;
		synchronized (q3Client) {
		    response = q3Client.getInfo("xxxxgetstatus");
		}

		//  take care of a shutdown broadcast
		if(response != null && response.indexOf("shutdown") > 0) {
		    q3Gui.effectsPanel.setFadeText("Server restarting.");
		    response = null;
		}

		//  make sure this is a status response
		if(response != null && response.indexOf("statusResponse") < 0)
		    response = null;
		
		//  yawn...
		try {
		    Thread.sleep(delay);
		} catch (InterruptedException ie) {
		    db("Trouble sleeping.");
		}
	    }

	    //  announce when we finally find a server
	    if(waiting) {
		q3Gui.effectsPanel.setFadeText("Found server: " 
					       + q3Client.host);
		waiting = false;
	    }

	    //  get general server info
	    map = response.substring((response.indexOf("mapname") + 8));
	    map = map.substring(0, map.indexOf('\\'));
	    maxClients 
		= response.substring(response.indexOf("sv_maxclients") + 14);
	    maxClients = maxClients.substring(0, maxClients.indexOf('\\'));
	    serverName 
		= response.substring(response.indexOf("sv_hostname") + 12);
	    serverName = serverName.substring(0, serverName.indexOf('\\'));
	    
	    timeLimit 
		= response.substring(response.indexOf("timelimit") + 10);
	    timeLimit = timeLimit.substring(0, timeLimit.indexOf('\\'));
	    fragLimit 
		= response.substring(response.indexOf("fraglimit") + 10);
	    fragLimit = fragLimit.substring(0, fragLimit.indexOf('\\'));

	    //  get the players
	    players = new Vector();
	    line = response;
	    response = response.substring(response.indexOf("\n") + 1);
	    response = response.substring(response.indexOf("\n") + 1);
	    while(response.indexOf("\n") > 0) {
		line = response.substring(0, response.indexOf("\n") + 1);
		response = response.substring(response.indexOf("\n") + 1);
		playerLine = line.trim();
		//  get score
		score = playerLine.substring(0, playerLine.indexOf(' ') + 1);
		playerLine = playerLine.substring(playerLine.indexOf(' ') + 1);
		//  get ping
		ping = playerLine.substring(0, playerLine.indexOf(' ') + 1);
		playerLine = playerLine.substring(playerLine.indexOf(' ') + 1);
		//  get the player name
		name = playerLine;
		name = name.replace('"', ' ').trim();
		//  create the player
		player = new Player();
		player.setName(name);
		player.setScore(score);
		player.setPing(ping);
		//  add the player by score
		if(players.isEmpty()) 
		    players.addElement(player);
		else {
		    for(int i = 0; i < players.size(); i++) {
			nextPlayer = (Player) players.elementAt(i);
			if(player.getScore() > nextPlayer.getScore()) {
			    players.insertElementAt(player, i);
			    break;
			} else {
			    if(i == (players.size() - 1)) {
				players.addElement(player);
				break;
			    }
			}
		    }
		}
	    }

	    //  check the server activity
	    if(players.isEmpty()) {
		//  only do this the first time
		if(!empty) {
		    q3Gui.effectsPanel.setText("Server empty.");
		    q3Gui.changeLogo(0, false, false);
		}
		empty = true;
		foundRealPlayer = false;
		active = false;
	    } else {
		if(((Player) players.elementAt(0)).getScore() 
		   >= Integer.parseInt(fragLimit.trim())
		   &&  !fragLimit.equals("0")) {
		    //  check to make sure there IS a fraglimit
		    q3Gui.changeLogo(players.size(), false, false);
		    winner = (Player) players.elementAt(0);
		    //  only do this the first time
		    if(active) {
			q3Gui.effectsPanel.setText(winner.getName() 
						   + " has won the game.");
			statistics.gameOver();
		    }
		    active = false;
		} else {
		    //  reset flag
		    foundRealPlayer = false;
		    for(int i = 0; i < players.size(); i++) {
			player = (Player) players.elementAt(i);
			name = player.getName();
			if(!q3Gui.botsPanel.botModel.contains(name)) {
			    //  update stats
			    if(!playerHistory.contains(player)) {
				statistics.newConnection();
			    }
			    foundRealPlayer = true;
			}
		    }
		    
		    q3Gui.changeLogo(players.size(), true, foundRealPlayer);
		    
		    //  only do this the first time
		    if(!active) {
			q3Gui.effectsPanel.setFadeText("New game started...");
			statistics.newGame();
		    }
		    active = true;
		}       
		empty = false;
	    } 

	    //  send stats whether we found any real people
	    statistics.setOccupied(foundRealPlayer);

	    //  send the players to the statistics
	    statistics.setPlayers(players);
	    statistics.setActive(active);

	    //  update the server info panel
	    if (!serverInfo.contains(serverName)) {
		q3Gui.serverPanel.setServerName(serverName);
		serverInfo.insertElementAt(serverName, 0);
		newInfo = true;
	    }
	    if (!serverInfo.contains(map)) {
		q3Gui.serverPanel.setMap(map);
		serverInfo.insertElementAt(map, 1);
		newInfo = true;
	    }
	    if (!serverInfo.contains(maxClients)) {
		q3Gui.serverPanel.setMaxClients(maxClients);
		serverInfo.insertElementAt(maxClients, 2);
		newInfo = true;
	    }
	    if (!serverInfo.contains(timeLimit)) {
		q3Gui.serverPanel.setTimelimit(timeLimit);
		serverInfo.insertElementAt(timeLimit, 3);
		newInfo = true;
	    }
	    if (!serverInfo.contains(fragLimit)) {
		q3Gui.serverPanel.setFraglimit(fragLimit);
		serverInfo.insertElementAt(fragLimit, 4);
		newInfo = true;
	    }
	    if(newInfo) {
		try {
		    SwingUtilities.invokeLater(q3Gui.serverPanel);
		} catch (Exception e) {
		    db("Problem updating gui...");
		}
		newInfo = false;
	    }

	    q3Gui.playerDisplay.setPlayers(players);

	    //  update the history
	    playerHistory = players;
	    
	    //  perform system cleanup
	    System.runFinalization();
	    System.gc();
	    
	    //  yawn...
	    try {
		Thread.sleep(delay);
	    } catch (InterruptedException ie) {
		db("Trouble sleeping.");
	    }
	}
    }

    public void db(String s) {
	System.err.println(s);
    }

}
