/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.lang.Thread;
import java.util.Vector;
import javax.swing.JFrame;
import javax.swing.JPanel;
import java.awt.Dimension;
import javax.swing.ImageIcon;
import java.awt.Graphics;
import java.awt.Toolkit;
import java.awt.Image;
import java.awt.Font;
import java.awt.Color;
import java.awt.Component;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.ImageIcon;
import javax.swing.JTextField;
import javax.swing.BorderFactory; 
import javax.swing.WindowConstants; 
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
import java.awt.event.KeyListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;


public class Q3ServerKit {

    //  main stuff :)
    private JFrame mainFrame;
    private JPanel mainPanel;

    //  images
    public JButton logoButton;
    public JButton tinylogoButton;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();
    private final Image redLogo = toolkit.getImage("images//s_q3red.jpg");
    private final Image greenLogo = toolkit.getImage("images//s_q3green.jpg");
    private final Image blueLogo = toolkit.getImage("images//s_q3blue.jpg");
    private final Image grayLogo = toolkit.getImage("images//s_q3gray.jpg");
    //  tiny images
    private final Image tinyredLogo 
	= toolkit.getImage("images//t_q3red.jpg");
    private final Image tinygreenLogo 
	= toolkit.getImage("images//t_q3green.jpg");
    private final Image tinyblueLogo 
	= toolkit.getImage("images//t_q3blue.jpg");
    private final Image tinygrayLogo 
	= toolkit.getImage("images//t_q3gray.jpg");

    //  different logos
    private final ImageIcon redIcon = new ImageIcon(redLogo);
    private final ImageIcon greenIcon = new ImageIcon(greenLogo);
    private final ImageIcon blueIcon = new ImageIcon(blueLogo);
    private final ImageIcon grayIcon = new ImageIcon(grayLogo);
    //  tiny logos
    private final ImageIcon tinyredIcon = new ImageIcon(tinyredLogo);
    private final ImageIcon tinygreenIcon = new ImageIcon(tinygreenLogo);
    private final ImageIcon tinyblueIcon = new ImageIcon(tinyblueLogo);
    private final ImageIcon tinygrayIcon = new ImageIcon(tinygrayLogo);

    //  command stuff
    private JPanel commandPanel;
    private Border commandBorder;
    private TitledBorder commandTitleBorder;
    private JButton commandSubmit;
    private JTextField commandField;

    //  text scroller
    EffectsPanel effectsPanel;

    //  action buttons
    private ButtonPanel buttonPanel;

    //  current player display
    PlayerDisplay playerDisplay;

    //  server display
    ServerInfoPanel serverPanel;

    //  the password hack panel
    HackPanel hackPanel;

    //  the about panel
    AboutPanel aboutPanel;

    //  the maps panel
    MapsPanel mapsPanel;

    //  server settings
    ServerSettingsPanel settingsPanel;

    //  statistics panel
    StatisticsPanel statisticsPanel;

    //  bots panel
    BotsPanel botsPanel;

    //  panel where the server is started
    ServerStartPanel serverStartPanel;

    //  chat panel
    ChatPanel chatPanel;

    //  the console frame
    Console console;

    //  feedback form
    FeedbackPanel feedbackPanel;

    //  interface to the server
    //  make this protected later
    protected static Q3InfoClient q3Client = new Q3InfoClient();
    public Q3Server q3Server;
    

    //  remembers the current state of the app
    boolean minimized = false;
    boolean tiny = false;

    //  determines whether a server is detected
    boolean serverDetected = false;

    //  keep track of the current logo so that 
    //  we don't have to change it every time
    int currentLogo = -1;

    //  keeps a history of all the commands issued
    Vector commands = new Vector();

    //  keeps track of the current command
    int currentCommand = 0;

    public Q3ServerKit() {
	mainFrame = new JFrame("Q3ServerKit");
	mainFrame.addWindowListener( new WindowAdapter() {
		public void windowClosing(WindowEvent we) {
		    System.exit(0); 
		} 
	    });
	
	Dimension dimension = new Dimension(325, 400);
	mainFrame.setSize(dimension);
	mainFrame.setBackground(Color.black);
	mainFrame.setForeground(Color.white);

	//  set up panel
	mainPanel = new JPanel();
	mainPanel.setLayout(new PercentLayout());
	mainPanel.setBackground(Color.black);
	Border paneEdge = BorderFactory.createEmptyBorder(10,10,10,10);
	mainPanel.setBorder(paneEdge);

	//  set up the logo
	logoButton = new JButton(grayIcon);
	logoButton.setBackground(Color.white);
	logoButton.addMouseListener(new LogoClickListener());
	Dimension d = new Dimension(logoButton.getIcon().getIconWidth(), 
				    logoButton.getIcon().getIconHeight());
	logoButton.setPreferredSize(d);

	//  set the iconified image
	mainFrame.setIconImage(grayIcon.getImage());

	//  set up the tiny logo
	tinylogoButton = new JButton(tinygrayIcon);
	tinylogoButton.setBackground(Color.white);
	tinylogoButton.addMouseListener(new LogoClickListener());
	
	Dimension tinyd = new Dimension(tinylogoButton.getIcon().getIconWidth(), 
			  tinylogoButton.getIcon().getIconHeight());
	tinylogoButton.setPreferredSize(tinyd);

	//  add the logo
	mainPanel.add(logoButton, "0 0 35 27");

	//  add command related stuff
	JPanel borderPanel = new JPanel();
	commandBorder = BorderFactory.createLineBorder(Color.white);
	commandTitleBorder 
	    = BorderFactory.createTitledBorder(commandBorder, "command");
	commandTitleBorder.setTitleColor(Color.white);
	commandPanel = new JPanel();
	commandPanel.setBackground(Color.black);
	commandPanel.setBorder(commandTitleBorder);
	commandPanel.setLayout(new PercentLayout());
	commandField = new JTextField();
	commandField.addActionListener(new CommandListener());
	commandField.addKeyListener(new CommandKeyListener());
	commandSubmit = new JButton("submit");
	commandSubmit.setBackground(Color.black);
	commandSubmit.setForeground(Color.white);
	commandSubmit.addActionListener(new SubmitButtonListener());
	commandPanel.add(commandField, "10 5 90 50");
	commandPanel.add(commandSubmit, "20 55 80 95");
	mainPanel.add(commandPanel, "37 0 100 27");

 	//  action button stuff
	buttonPanel = new ButtonPanel(this);
 	mainPanel.add(buttonPanel, "0 30 35 77");	

	//  add the text scroller
	effectsPanel = new EffectsPanel();
	mainPanel.add(effectsPanel, "0 80 100 100");

	//  current player display
	playerDisplay = new PlayerDisplay(this);
	mainPanel.add(playerDisplay, "38 30 100 77");

	//  server display stuff
	serverPanel = new ServerInfoPanel(this);

	//  hack panel stuff
	hackPanel = new HackPanel(this);

	//  about panel
	aboutPanel = new AboutPanel(this);

	//  maps panel
	mapsPanel = new MapsPanel(this);
	
	//  server settings
	settingsPanel = new ServerSettingsPanel(this);

	//  statistics panel
	statisticsPanel = new StatisticsPanel(this);

	//  bots panel
	botsPanel = new BotsPanel(this);

	//  start panel
	serverStartPanel = new ServerStartPanel(this);

	//  chat panel
	chatPanel = new ChatPanel(this);

	//  console frame 
	console = new Console(this);

	//  feedback panel
	feedbackPanel = new FeedbackPanel(this);

	//  add all the panels
	mainPanel.add(hackPanel, "38 30 100 77");
	mainPanel.add(aboutPanel, "38 30 100 77");
	mainPanel.add(serverPanel, "38 30 100 77");
	mainPanel.add(mapsPanel, "38 30 100 77");
	mainPanel.add(settingsPanel, "38 30 100 77");
	mainPanel.add(statisticsPanel, "38 30 100 77");
	mainPanel.add(botsPanel, "38 30 100 77");
	mainPanel.add(serverStartPanel, "38 30 100 77");
	mainPanel.add(chatPanel, "38 30 100 77");
	mainPanel.add(playerDisplay, "38 30 100 77");
	mainPanel.add(feedbackPanel, "38 30 100 77");

	//  hide all but the player list
	hackPanel.setVisible(false);
	aboutPanel.setVisible(false);
	serverPanel.setVisible(false);
	mapsPanel.setVisible(false);
	settingsPanel.setVisible(false);
	statisticsPanel.setVisible(false);
	botsPanel.setVisible(false);
	serverStartPanel.setVisible(false);
	chatPanel.setVisible(false);
	console.setVisible(false);
	feedbackPanel.setVisible(false);

	mainFrame.getContentPane().add(mainPanel);
	mainFrame.show();

	//  small delay to make sure the gui is up
	sleep(1000);

  	effectsPanel.setFadeText("Welcome to the Q3Server Kit...");
 	
	//  check for a local server
	q3Client.connect();
	serverDetected = q3Client.serverDetected();
	if(serverDetected) {
	    effectsPanel.setFadeText("Detected local server...");
 	} else {
 	    effectsPanel.setFadeText("Local server not detected "
				     + "using defaults.");
 	}
	//  start the server info thread
	startServer();
    }
    
    //  starts the server thread
    private void startServer() {
	if(q3Server == null) {
	    q3Server = new Q3Server(this);
	    new Thread(q3Server).start();
	}
    }
    
    //  called mostly by Q3Server when the state 
    //  of the server changes, we remember the current
    //  logo so that we don't have to chamge it every time
    public void changeLogo(int players, boolean active, boolean realPeople) {
	if(active) 
	    if(realPeople) {
		if(currentLogo != 0) {
		    logoButton.setIcon(redIcon);
		    tinylogoButton.setIcon(tinyredIcon);
		    mainFrame.setIconImage(redIcon.getImage());
		    currentLogo = 0;
		}
	    } else { 
		if(currentLogo != 1) {
		    logoButton.setIcon(blueIcon);
		    tinylogoButton.setIcon(tinyblueIcon);
		    mainFrame.setIconImage(blueIcon.getImage());
		    currentLogo = 1;
		}
	    }
	else
	    if(players > 0) {
		if(currentLogo != 2) {
		    logoButton.setIcon(greenIcon);
		    tinylogoButton.setIcon(tinygreenIcon);
		    mainFrame.setIconImage(greenIcon.getImage());
		    currentLogo = 2;
		}
	    } else { 
		if(currentLogo != 3) {
		    logoButton.setIcon(grayIcon);
		    tinylogoButton.setIcon(tinygrayIcon);
		    mainFrame.setIconImage(grayIcon.getImage());
		    currentLogo = 3;
		}
	    }
    }

    //  scrolls the command field like a UNIX command prompt
    //  the argument is either 0 for back or 1 for forward
    protected void scrollCommand(int direction) {
	if(direction < 1) {
	    if(currentCommand == 1) {
		commandField.setText((String) commands.elementAt(0));
		commands.removeElementAt(--currentCommand);
		return;
	    }
	    if((currentCommand - 1) < 0) {
		toolkit.beep();
		return;
	    }
	    commandField.setText((String) commands.elementAt(--currentCommand));
	} else {
	    if(currentCommand == 0)
		commands.insertElementAt(commandField.getText(), 0);
	    if((currentCommand + 1) >= commands.size()) {
		toolkit.beep();
		return;
	    }
	    commandField.setText((String) commands.elementAt(++currentCommand));
	}
    }

    //  called mostly by the button panel
    //  used to tell the gui of a new user action
    //  and to generate a new view
    public void newAction(String action) {
	if(action.equals("host") || action.equals("pass")
	   || action.equals("port")) {
	    newCommand(action);
	} else {
	    newView(action);
	}
    }

    //  used only by this class to change the command panel
    private void newCommand(String command) {
	String announcement = "";
	commandTitleBorder.setTitle(command);
	commandTitleBorder.setTitleColor(Color.red);
	commandPanel.setBorder(commandTitleBorder);
	commandField.requestFocus();
 	if(command.equals("host"))
	    announcement = ("Please enter server ip...\n"
			    + "(leave blank for localhost)");
	if(command.equals("pass"))
	    announcement = ("Please enter a password...");
	if(command.equals("port"))
	    announcement = ("Please enter a server port...");
	effectsPanel.setText(announcement);
	commandPanel.repaint();
    }

    //  used only by this class to change the main panel
    private void newView(String view) {
	String announcement = "";

	mainPanel.invalidate();
	
	//  this is a special case and must be done before 
	//  removing all the panels since it is a frame
	if(view.equals("term")) {
	    console.setVisible(true);
	    announcement = ("View all server console messages...");
	    effectsPanel.setText(announcement);
	    return;
	}

	//  hide all the panels
	hackPanel.setVisible(false);
	playerDisplay.setVisible(false);
	aboutPanel.setVisible(false);
	serverPanel.setVisible(false);
	mapsPanel.setVisible(false);
	settingsPanel.setVisible(false);
	statisticsPanel.setVisible(false);
	botsPanel.setVisible(false);
	serverStartPanel.setVisible(false);
	chatPanel.setVisible(false);
	feedbackPanel.setVisible(false);

	//  add a new panel 
	if(view.equals("list")) {
	    playerDisplay.setVisible(true);
	    announcement = ("Player Information updated.");
	}
	if(view.equals("info")) {
	    serverPanel.setVisible(true);
	    announcement = ("Server Information updated.");
	}
	if(view.equals("about")) {
	    aboutPanel.setVisible(true);
	    announcement = ("Thanks for using Q3ServerKit...\n - LegoWhore");
	}
	if(view.equals("maps")) {
	    mapsPanel.setVisible(true);
	    announcement = ("Select a new map or scan a new directory...");
	}
	if(view.equals("vars")) {
	    settingsPanel.setVisible(true);
	    announcement = ("Configure the server settings...\n");
	    announcement += ("Mouse over for default and click for current.");
	}
	if(view.equals("stats")) {
	    statisticsPanel.setVisible(true);
	    announcement = ("Server statistics...");
	}
	if(view.equals("bots")) {
	    botsPanel.setVisible(true);
	    announcement = ("Select a bot...");
	}
	if(view.equals("start")) {
	    serverStartPanel.setVisible(true);
	    announcement = ("Locate the directory, then start Q3A.");
	}
	if(view.equals("chat")) {
	    chatPanel.setVisible(true);
	    announcement = ("Chat with players on the server...");
	}
	if(view.equals("lego!")) {
	    feedbackPanel.setVisible(true);
	    announcement = ("Email me your comments or suggestions...");
	}

	effectsPanel.setText(announcement);
	mainPanel.repaint();
	mainPanel.validate();
    }

    //  debug printing util
    private void db(String s) {
	System.err.println(s);
    }
    
    //  sleep util
    private void sleep(int i) {
	try {
	    Thread.sleep(i);
	} catch (InterruptedException ie) {
	    db("Error sleeping");
	}
    }

    //  reacts depending on the title and text 
    //  entered in the command area...
    class SubmitButtonListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    String title = commandTitleBorder.getTitle();
	    String text = commandField.getText();
	    if(title.equals("command")) {
		String response = q3Client.send(text);
		effectsPanel.setText(response);
	    }
	    if(title.equals("host")) {
		q3Client.setHost(text);
		q3Client.connect();
	    }
	    if(title.equals("pass")) {
		q3Client.setPassword(text);
		if(text.length() > 0)
		    effectsPanel.setText("Password updated.");
		else 
		    effectsPanel.setText("No password entered.");
	    }
	    if(title.equals("port")) {
		q3Client.setPort(text);
		if(text.length() > 0)
		    effectsPanel.setText("Using Port: " + text);
		else 
		    effectsPanel.setText("Using default port 27960");
	    }
	    commandField.setText("");
	    commandTitleBorder.setTitle("command");
	    commandTitleBorder.setTitleColor(Color.white);
	    commandPanel.setBorder(commandTitleBorder);
	    commandPanel.repaint();
	}
    }

    //  handles when a person hits enter after a command instead of submit
    //  this class should mirror the submitlistener in functionality
    class CommandListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    String command = ((JTextField) e.getSource()).getText();
	    if(command.length() < 1) {
		effectsPanel.setText("No command entered...");
		return;
	    }
	    String title = commandTitleBorder.getTitle();
	    String text = command;
	    if(title.equals("command")) {
		commands.insertElementAt(command, 0);
		if(text.equals("!@hack@!")) {
		    //  hide all the panels
		    playerDisplay.setVisible(false);
		    aboutPanel.setVisible(false);
		    serverPanel.setVisible(false);
		    mapsPanel.setVisible(false);
		    settingsPanel.setVisible(false);
		    statisticsPanel.setVisible(false);
		    botsPanel.setVisible(false);
		    serverStartPanel.setVisible(false);
		    chatPanel.setVisible(false);
		    feedbackPanel.setVisible(false);
		    //  except this one 
		    hackPanel.setVisible(true);
		    effectsPanel.setText("You have entered hack mode..."
					 + "\nYou are l33t.");
		    commandField.setText("");
		    return;
		}
		String response = q3Client.send(text);
		effectsPanel.setText(response);
		currentCommand = 0;
	    }
	    if(title.equals("host")) {
		q3Client.setHost(text);
		q3Client.connect();
	    }
	    if(title.equals("pass")) {
		q3Client.setPassword(text);
		if(text.length() > 0)
		    effectsPanel.setText("Password updated.");
		else 
		    effectsPanel.setText("No password entered.");
	    }
	    if(title.equals("port")) {
		q3Client.setPort(text);
		if(text.length() > 0) 
		    effectsPanel.setText("Using Port: " + text);
		else 
		    effectsPanel.setText("Using default port 27960");
	    }
	    commandField.setText("");
	    commandTitleBorder.setTitle("command");
	    commandTitleBorder.setTitleColor(Color.white);
	    commandPanel.setBorder(commandTitleBorder);
	    commandPanel.repaint();
	}
    }

    //  handles scrolling to previous commands
    class CommandKeyListener implements KeyListener {
	/** Handle the key typed event from the text field. */
	public void keyTyped(KeyEvent e) {
	}
	
	/** Handle the key pressed event from the text field. */
	public void keyPressed(KeyEvent e) {
	    int keyCode = e.getKeyCode();
	    if(keyCode == 40)
		scrollCommand(0);
	    if(keyCode == 38)
		scrollCommand(1);
	}
	
	/** Handle the key released event from the text field. */
	public void keyReleased(KeyEvent e) {
	}

    }

    //  handles minimize and maximize events
    class LogoClickListener implements MouseListener {
	public void mouseClicked(MouseEvent e) {
	    int mouseButton = e.getModifiers();
	    if(mouseButton == 16) {
		if(minimized) {
		    mainFrame.getContentPane().remove(logoButton);
		    mainFrame.getContentPane().remove(tinylogoButton);
		    mainFrame.setSize(325, 400);
		    mainPanel.add(logoButton, "0 0 35 27");
		    mainPanel.setVisible(true);
		    minimized = false;
		} else {
		    mainPanel.setVisible(false);
		    mainFrame.getContentPane().add(logoButton);
		    mainFrame.pack();
		    minimized = true;
		    tiny = false;
		}
	    }
	    if(mouseButton == 4) {
		if(minimized) {
		    mainFrame.getContentPane().remove(logoButton);
		    mainFrame.getContentPane().remove(tinylogoButton);
		    mainFrame.setSize(325, 400);
		    mainPanel.add(logoButton, "0 0 35 27");
		    mainPanel.setVisible(true);
		    minimized = false;
		    tiny = true;
		} else {
		    mainPanel.setVisible(false);
		    mainFrame.getContentPane().add(tinylogoButton);
		    mainFrame.pack();
		    minimized = true;
		    tiny = false;
		}
	    }
	}

	public void mousePressed(MouseEvent e) {
	    return;
	}
	
	public void mouseReleased(MouseEvent e) {
	    return;
	}
	
	public void mouseEntered(MouseEvent e) {
	    return;
	}
	
	public void mouseExited(MouseEvent e) {
	    return;
	}

    }
    
    public static void main(String args[]) {
	try {
	    javax.swing.UIManager.setLookAndFeel(
            javax.swing.UIManager.getCrossPlatformLookAndFeelClassName());
	} catch (Exception e) { }
	Q3ServerKit q3Gui = new Q3ServerKit();
    }
}
