/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.io.*;

public class ServerProcess extends Thread {

    //  the path to quake3
    String path;

    //  the config file name
    String config;

    Runtime run;
    Process process;
    InputStream input;
    InputStreamReader inputReader;
    BufferedReader reader;

    //  a flag used to stop the server
    boolean stop = false;

    Q3ServerKit q3Gui;

    public ServerProcess(Q3ServerKit q3Gui) {
	this.q3Gui = q3Gui;
	run = Runtime.getRuntime();
    }

    //  this command starts the server
    public void executeCommand(String command) {
	try {
	    process = run.exec(command);
	} catch (IOException ioe) {
	    ioe.printStackTrace();
	}
    }

    //  reads and parses the server output
    public void run() {
	inputReader 
	    = new InputStreamReader(process.getErrorStream());
	reader = new BufferedReader(inputReader);
	while(true) {
	    if(stop) {
		//  stop q3 and exit run
		db("stopping q3");
		process.destroy();
		return;
	    }
	    try {
		boolean ready = reader.ready();
		//  if there's something to read...read it
		if(ready) {
		    String line = reader.readLine();
		    q3Gui.console.addText(line);
		    if(line.startsWith("say:")) {
			line = line.substring(line.indexOf(':') + 1);
			q3Gui.chatPanel.addText(line);
		    }
		} else {
		    //  yawn...
		    try {
			Thread.sleep(1000);
		    } catch (InterruptedException ie) {
			db("Trouble sleeping.");
		    }
		}
	    } catch (IOException ioe) {
		ioe.printStackTrace();
	    }
	}
    }
    
    //  this method is called to actually start the server
    protected void startServer(String q3Path, String config, 
			       boolean privateServer) {
	this.path = q3Path;
	this.config = config;
	int privateServerFlag = 0;
	if(privateServer)
	    privateServerFlag = 1;
	else
	    privateServerFlag = 2;
	//  check if the thread is active 
	//  if so, stop it and wait for it to die
	if(this.isAlive()) {
	    stop = true;
	    try {
		join();
	    } catch (InterruptedException ie) {
		db("could not stop server");
		return;
	    }
	    stop = false;
	}
 	executeCommand(path + " +set dedicated " + privateServerFlag 
 		       + " +exec " + config);
 	this.start();
    }

    private void db(String s) {
	System.err.println(s);
    }
}
