/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.awt.Component;
import java.awt.Color;
import javax.swing.JPanel;
import javax.swing.JButton;
import java.awt.Dimension;
import java.util.Vector;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.SwingUtilities;

public class ServerSettingsPanel extends JPanel implements Runnable {

    Q3ServerKit q3Gui;
    JButton moreButton;
    JButton submitButton;
    Vector settings;

    int nextPanel = 0;

    Vector currentPanels;

    public ServerSettingsPanel(Q3ServerKit q3Gui) {
	this.q3Gui = q3Gui;
	setLayout(new PercentLayout());
	setBackground(Color.black);
	settings = new Vector();
	currentPanels = new Vector();
	submitButton = new JButton("submit");
	submitButton.setBackground(Color.black);
	submitButton.setForeground(Color.white);
	submitButton.addActionListener(new SubmitListener());
	moreButton = new JButton("more");
	moreButton.setBackground(Color.black);
	moreButton.setForeground(Color.white);
	moreButton.addActionListener(new MoreListener());
	new Settings(this);
	nextPanel();
    }

    //  called by SettingPanel to add itself to the list
    //  most of the classes extending settingPanel are in 
    //  Settings.java
    public void addSetting(SettingPanel setting) {
	settings.addElement(setting);
    }

    //  calls the event dispatch thread to change the panel
    private void nextPanel() {
	try {
	    SwingUtilities.invokeLater(this);
	} catch (Exception e) {
	    db("Problem updating settings panel...");
	}
    }

    //  run in the event dispatch thread...
    //  used to change the panels
    public void run() {
	invalidate();
	int size = settings.size();
	int top = 0;
	int bottom = 27;
	int panels = 0;
	this.removeAll();
	currentPanels.clear();
	while(panels < 3) {
	    String layout = "0 " + top + " 100 " + bottom;
	    //  check if we're at the end of the panels
	    if(nextPanel > (size - 1)) 
		nextPanel = 0;
	    JPanel panel = (JPanel) settings.elementAt(nextPanel);
	    add(panel, layout);
	    currentPanels.addElement(panel);
	    top = bottom + 2;
	    bottom = top + 27;
	    panels++;
	    nextPanel++;
	}
	add(submitButton, "0 87 48 100");
	add(moreButton, "52 87 100 100");
	repaint();
	validate();
    }

    //  listens for a 'more' request
    public class MoreListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    nextPanel();
	}

    }

    //  listens for a 'submit' request
    public class SubmitListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    int size = currentPanels.size();
	    for(int i = 0; i < size; i++) {
		SettingPanel settingPanel 
		    = (SettingPanel) currentPanels.elementAt(i);
		settingPanel.setValue();
	    }
	}

    }

    private void db(String s) {
	System.err.println(s);
    }
    
}
