/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.awt.Color;
import javax.swing.JPanel;
import javax.swing.JLabel;
import javax.swing.JTextField;
import javax.swing.JButton;
import javax.swing.BorderFactory; 
import javax.swing.border.Border;
import java.awt.Font;
import java.awt.event.ActionListener;
import java.awt.event.MouseListener;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import javax.swing.SwingUtilities;

public abstract class SettingPanel extends JPanel implements Runnable {

    public static int number = 0;

    JLabel settingLabel;
    JLabel descriptionLabel;
    JTextField settingText;

    String settingType = new String("Type: " + number);
    String description = new String("A description of this setting.");
    String defaultValue = new String("value");

    String command = new String("command");
    String currentValue;

    Font smallFont;

    private Border settingBorder;
    ServerSettingsPanel parent;

    public SettingPanel(ServerSettingsPanel parent) {
	this.parent = parent;
	Border etched = BorderFactory.createLineBorder(Color.white);
	setBorder(etched);
	setLayout(new PercentLayout());
	smallFont = new Font("plain", Font.PLAIN, 10);
	setBackground(Color.black);
	addMouseListener(new ClickListener());
    }

    //  creates the panel and adds it to the parent
    protected void createPanel() {
	settingLabel = new JLabel(settingType);
	settingLabel.setForeground(Color.white);
	descriptionLabel = new JLabel(description);
	descriptionLabel.setFont(smallFont);
	descriptionLabel.setForeground(Color.white);
	settingText = new JTextField();
	settingText.setHorizontalAlignment(JTextField.RIGHT);
	settingText.addActionListener(new EnterListener());
	add(settingLabel, "2 5 60 59");
	add(descriptionLabel, "2 60 98 100");
	add(settingText, "61 7 98 59");
	setToolTipText("Default: " + defaultValue);
	parent.addSetting(this);
	number++;
    }

    //  grabs the text and sends this 
    //  command off to the server
    protected void setValue() {
	String value = settingText.getText();
	if(value.length() < 1) {
	    parent.q3Gui.effectsPanel.setText("No value entered...");
	    return;
	}
	String response = parent.q3Gui.q3Client.send(command + " " + value);
	if(response == null) {
	    parent.q3Gui.effectsPanel.setText("No server detected...");
	    return;
	}
	if(response.startsWith("Bad")) {
	    parent.q3Gui.effectsPanel.setText("Incorrect password...");
	    //  switch to the password command...
	    parent.q3Gui.newAction("pass");
	    return;
	}
	currentValue = value;
	settingText.setText(currentValue);
	parent.q3Gui.effectsPanel.setText(response);
    }

    //  listens for when the user hits enter on a setting
    public class EnterListener implements ActionListener {
	public void actionPerformed(ActionEvent e) {
	    setValue();
	}
    }

    //  run in the event dispatch thread to update
    //  the current value of this variable from the server
    public void run() {
	settingText.setText(currentValue);
    }

    //  used to set the current value
    public void updateValue() {
	String response = parent.q3Gui.q3Client.send(command);
	if(response == null) {
	    parent.q3Gui.effectsPanel.setText("No server detected...");
	    return;
	}
	if(response.startsWith("Bad")) {
	    parent.q3Gui.effectsPanel.setText("Incorrect password...");
	    //  switch to the password command...
	    parent.q3Gui.newAction("pass");
	    return;
	}
	if(response.indexOf(command) < 0) {
	    parent.q3Gui.effectsPanel.setText(response);
	    return;
	}
	String current = response.substring(response.indexOf("is:") + 4);
	current = current.substring(0, current.indexOf("^"));
	currentValue = current;
	try {
	    SwingUtilities.invokeLater(this);
	} catch (Exception e) {
	    db("Problem updating setting panel...");
	}
    }

    //  retrieves the current setting
    class ClickListener implements MouseListener {
	public void mouseClicked(MouseEvent e) {
	    updateValue();
	}

	public void mousePressed(MouseEvent e) {
	    return;
	}
	
	public void mouseReleased(MouseEvent e) {
	    return;
	}
	
	public void mouseEntered(MouseEvent e) {
	    return;
	}
	
	public void mouseExited(MouseEvent e) {
	    return;
	}
    }

    private void db(String s) {
	System.err.println(s);
    }
    
}
