/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

//  just a place to group all the settings
public class Settings {

    ServerSettingsPanel parent;

    public Settings(ServerSettingsPanel parent) {
	this.parent = parent;
	new GravitySetting(parent);
	new KnockbackSetting(parent);
	new SpeedSetting(parent);
	new TimeoutSetting(parent);
	new MaxClientsSetting(parent);
	new InactivitySetting(parent);
	new BloodSetting(parent);
	new QuadFactorSetting(parent);
	new MaxRateSetting(parent);
	new AllowDownloadSetting(parent);
	new FpsSetting(parent);
	new ForceRespawnSetting(parent);
	new BotNoChatSetting(parent);
	new BotGroundOnlySetting(parent);
	new ZombieSetting(parent);
	new FraglimitSetting(parent);
	new TimelimitSetting(parent);
	new GametypeSetting(parent);
    }

    //  gravity of the server
    class GravitySetting extends SettingPanel {

	public GravitySetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "gravity";
	    command = "g_gravity";
	    description = "the gravity of the server.";
	    defaultValue = "800";
	    createPanel();
	}
    }
    
    //  how much a hit knocks a player back
    class KnockbackSetting extends SettingPanel {
	
	public KnockbackSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "knockback";
	    command = "g_knockback";
	    description = "how much a hit knocks a player.";
	    defaultValue = "1000";
	    createPanel();
	}
    }

    //  the maximum # of players allowed
    class MaxClientsSetting extends SettingPanel {
	
	public MaxClientsSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "maxclients";
	    command = "sv_maxclients";
	    description = "the maximum # of players.";
	    defaultValue = "8";
	    createPanel();
	}
    }
    
    //  the maximum speed a player can move
    class SpeedSetting extends SettingPanel {
	
	public SpeedSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "speed";
	    command = "g_speed";
	    description = "how fast a player can move.";
	    defaultValue = "320";
	    createPanel();
	}
    }
 
    //  how long before the server assumes a disconnect
    class TimeoutSetting extends SettingPanel {
	
	public TimeoutSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "timeout";
	    command = "sv_timeout";
	    description = "before assuming disconnect.";
	    defaultValue = "120";
	    createPanel();
	}
    }

    //  how long before a player is kicked due to inactivity
    class InactivitySetting extends SettingPanel {
	
	public InactivitySetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "inactivity";
	    command = "g_inactivity";
	    description = "time before player is kicked.";
	    defaultValue = "0";
	    createPanel();
	}
    }
    
    //  server setting for blood animations...
    //  set to '0' for optimization, '1' for gory as hell :)
    class BloodSetting extends SettingPanel {
	
	public BloodSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "blood";
	    command = "com_blood";
	    description = "toggle blood animations.";
	    defaultValue = "1";
	    createPanel();
	}
    }

    //  sets the how much the quad increases the damage
    //  although it's called quad the default in Q3A is 3
    class QuadFactorSetting extends SettingPanel {
	
	public QuadFactorSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "quad factor";
	    command = "g_quadfactor";
	    description = "set quad damage multiplier.";
	    defaultValue = "3";
	    createPanel();
	}
    }

    //  sets the max rate for each client so that someone 
    //  on a T3 doesn't suck up all the bandwidth
    class MaxRateSetting extends SettingPanel {
	
	public MaxRateSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "max rate";
	    command = "sv_maxRate";
	    description = "max bandwidth for each client.";
	    defaultValue = "0";
	    createPanel();
	}
    }

    //  determines whether clients can Download maps, skins, etc.
    class AllowDownloadSetting extends SettingPanel {
	
	public AllowDownloadSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "download";
	    command = "sv_allowdownload";
	    description = "toggle downloading maps, etc.";
	    defaultValue = "1";
	    createPanel();
	}
    }

    //  # of frames per second the server will send each client
    class FpsSetting extends SettingPanel {
	
	public FpsSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "fps";
	    command = "sv_fps";
	    description = "how many fps the server sends.";
	    defaultValue = "20";
	    createPanel();
	}
    }

    //  time before a client is forcibly respawned after a death
    class ForceRespawnSetting extends SettingPanel {
	
	public ForceRespawnSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "respawn";
	    command = "g_forcerespawn";
	    description = "time before a forced respawn.";
	    defaultValue = "20";
	    createPanel();
	}
    }

    //  whether the bots chat or not...
    class BotNoChatSetting extends SettingPanel {
	
	public BotNoChatSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "bot nochat";
	    command = "bot_nochat";
	    description = "whether the bots talk or not.";
	    defaultValue = "0";
	    createPanel();
	}
    }

    //  whether the bots jump or not...
    class BotGroundOnlySetting extends SettingPanel {
	
	public BotGroundOnlySetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "ground only";
	    command = "bot_groundonly";
	    description = "whether the bots jump or not.";
	    defaultValue = "1";
	    createPanel();
	}
    }
    
    //  time before a frozen player is kicked...
    class ZombieSetting extends SettingPanel {
	
	public ZombieSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "zombie time";
	    command = "sv_zombietime";
	    description = "minutes to drop frozen clients.";
	    defaultValue = "2";
	    createPanel();
	}
    }

    //  highest kill total allowed before the game is over
    class FraglimitSetting extends SettingPanel {
	
	public FraglimitSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "fraglimit";
	    command = "fraglimit";
	    description = "highest kill total allowed.";
	    defaultValue = "20";
	    createPanel();
	}
    }

    //  time before the game is over
    class TimelimitSetting extends SettingPanel {
	
	public TimelimitSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "timelimit";
	    command = "timelimit";
	    description = "minutes before the game ends.";
	    defaultValue = "20";
	    createPanel();
	}
    }
    //  time before the game is over
    class GametypeSetting extends SettingPanel {
	
	public GametypeSetting(ServerSettingsPanel parent) {
	    super(parent);
	    settingType = "gametype";
	    command = "g_gametype";
	    description = "Deathmatch, Teams, 1 on 1.";
	    defaultValue = "0";
	    createPanel();
	}
    }

}
