/*
 * Copyright (C) 2000 Free Software Foundation, Inc.
 * Brian Satterfield <legowhore@legowhore.com>, 2000.
 * Chris Garrett
 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

package com.legowhore.q3;

import java.awt.Color;
import javax.swing.JPanel;
import javax.swing.JTextPane;
import javax.swing.JScrollPane;
import javax.swing.JScrollBar;
import javax.swing.text.DefaultStyledDocument;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.BadLocationException;
import javax.swing.BorderFactory;
import javax.swing.SwingUtilities;

public class EffectsPanel extends JPanel implements Runnable {

    private JTextPane infoBox;
    private JScrollPane scrollPane;
    private boolean autoScroll = true;
    private String fadeText = null;
    private String text = null;
    private Color textColor;
    
    public EffectsPanel() {
	setLayout(new PercentLayout());
	setBorder(BorderFactory.createLineBorder(Color.white));
	infoBox = new JTextPane(new DefaultStyledDocument());
	infoBox.setForeground(Color.white);
	infoBox.setBackground(Color.black);
	infoBox.setEditable(false);
	
	scrollPane = new JScrollPane(infoBox);
	
	add("0 0 100 100", scrollPane);
	
    }
	
    public void setText(String text) {
	this.text = text;
	try {
	    SwingUtilities.invokeLater(this);
	} catch (Exception e) {
	    db("Problem updating effects panel...");
	}
    }
    
    public void setFadeText(String fadeText) {
	this.fadeText = fadeText;
	int shade = 255;
	textColor = new Color(shade, shade, shade);
	while(shade >= 5) {
	    try {
		SwingUtilities.invokeLater(this);
	    } catch (Exception e) {
		db("Problem updating effects panel...");
	    }
	    sleep(25);
	    shade = shade - 5;
	    textColor = new Color(shade, shade, shade);
	}
	this.fadeText = null;
    }
    
    public void run() {
	if(fadeText == null && text == null) {
	    return;
	} else { 
	    if(text != null) {
		replaceInfo(text, Color.white);
		text = null;
	    }
	    if(fadeText != null)
		replaceInfo(fadeText, textColor);
	}
    }

    //  debug printing util
    private void db(String s) {
	System.err.println(s);
    }
    
    //  sleep util
    private void sleep(int i) {
	try {
	    Thread.sleep(i);
	} catch (InterruptedException ie) {
	    db("Error sleeping");
	}
    }

    public void setAutoScroll(boolean scrollIt) {
	autoScroll = scrollIt;
    }
    
    public void replaceInfo(String info, Color color) {
	SimpleAttributeSet coloredFont = new SimpleAttributeSet();
	StyleConstants.setForeground(coloredFont, color);
	StyleConstants.setFontFamily(coloredFont, "SansSerif");
	StyledDocument doc = infoBox.getStyledDocument();
	
	try {
	    doc.remove(0, doc.getLength());
	    doc.insertString(doc.getLength(), info + "\n", coloredFont);
	    if(autoScroll) {
		JScrollBar bar = scrollPane.getVerticalScrollBar();
		if(bar != null)
		    bar.setValue(bar.getMaximum());
	    }
	    
	} catch(BadLocationException ble) {
	    
	    System.err.println("Caught Bad Location Exception (replaceInfo)\n");
	    
	}
	
    }
}

