package com.legowhore.q3;

//  this class will keep stats for the server
// 	- # of connections
// 		per day
// 		per month
// 		total
// 	- % of time occupied
// 	- # of games played
// 	- game length
// 	- average connecting ping
// 	- total # of frags (?)
// 	- list of winners (?)

import java.util.Vector;


public class Statistics implements Runnable {

    long totalSeconds;
    long startTime;
    long seconds;
    long minutes;
    long hours;
    long days;
    long weeks;
    
    boolean endGame = false;
    boolean newGame = false;
    boolean active = false;

    //  stats
    int connections = 0;
    int gamesPlayed = 0;
    boolean realPeople = false;
    int totalFrags = 0;
    double secondsOccupied = 0.0;
    double secondsUnoccupied = 0.0;


    Vector players;
    Vector playerHistory;

    StatisticsPanel statisticsPanel;

    public Statistics(Q3Server q3Server) {
	statisticsPanel = q3Server.q3Gui.statisticsPanel;
	startTime = System.currentTimeMillis();
    }

    protected void setPlayers(Vector players) {
	//  make sure that we don't add the difference between
	//  a players negative score and their initial zero at
	//  the beginning of a new game...
	if(endGame) {
	    return;
	}

	if(newGame) {
	    playerHistory =null;
	    newGame = false;
	}

	int size = players.size();
	Player player;
	Player oldPlayer;
	
	//  the first time we come up just
	//  add all frags... 
	if(playerHistory == null) {
	    for(int i = 0; i < size; i++) {
		player = (Player) players.elementAt(i);
		int score = player.getScore();
		if(score > 0)
		    totalFrags += score;
	    }
	    playerHistory = players;
	    return;
	}
	//  otherwise, only look at new frags
	for(int i = 0; i < size; i++) {
	    player = (Player) players.elementAt(i);
	    int position = playerHistory.indexOf(player);
	    
	    if(position >= 0) {
		oldPlayer = (Player) playerHistory.elementAt(position);
		int newFrags = player.getScore() - oldPlayer.getScore();
		if(newFrags > 0)
		    totalFrags += newFrags;
	    } else {
		totalFrags += player.getScore();
	    }
	}
	playerHistory = players;
	
    }

    protected void setOccupied(boolean realPeople) {
	this.realPeople = realPeople;
    }

    protected void setActive(boolean active) {
	this.active = active;
    }

    protected void newConnection() {
	connections++;
    }

    protected void gameOver() {
	endGame = true;
	gamesPlayed++;
    }

    protected void newGame() {
	endGame = false;;
	newGame = true;
    }

    protected String getUptime() {
	long currentTime = System.currentTimeMillis();
	totalSeconds = currentTime - startTime;
	long conversion = 1000;
	totalSeconds = totalSeconds/conversion;
	seconds = totalSeconds%60;
	long totalMinutes = totalSeconds/60;
	minutes = totalMinutes%60;
	long totalHours = totalMinutes/60;
	hours = totalHours%24;
	long totalDays = totalHours/24;
	days = totalDays%7;
	long totalWeeks = totalDays/7;
	weeks = totalWeeks%52;
	String time = new String();
	if(hours < 10)
	    time = "0" + hours;
	else 
	    time = "" + hours;
	if(minutes < 10)
	    time += ":0" + minutes;
	else 
	    time += ":" + minutes;
	if(seconds < 10)
	    time += ":0" + seconds;
	else 
	    time += ":" + seconds;
	
	String uptime = ("" + weeks + " weeks, " + days + " days " 
			 + "[" + time + "]");
	return uptime;
    }

    public void run() {
	long lastUpdate = System.currentTimeMillis();
	while(true) {
	    //  call this before the percent stuff below
	    statisticsPanel.setUptime(getUptime());
	    if(realPeople) {
		long currentSeconds = System.currentTimeMillis();
		secondsOccupied += (currentSeconds - lastUpdate)/1000;
	    } 
	    lastUpdate = System.currentTimeMillis();
	    double percentOccupied;
	    if(secondsOccupied == 0 || totalSeconds == 0)
		percentOccupied = 0;
	    else {
 		percentOccupied = secondsOccupied/totalSeconds;
	    }
	    percentOccupied = percentOccupied*100;
	    statisticsPanel.setFrags(totalFrags);
	    statisticsPanel.setGamesPlayed(gamesPlayed);
	    statisticsPanel.setConnections(connections);
	    statisticsPanel.setPercentOccupied(percentOccupied);
	    statisticsPanel.updatePanel();
	    //  yawn...
	    try {
		Thread.sleep(1000);
	    } catch (InterruptedException ie) {
		db("Trouble sleeping.");
	    }
	}
    }

    private void db(String s) {
	System.err.println(s);
    }
}
